﻿using HeuristicLab.Common;
using HeuristicLab.Encodings.SymbolicExpressionTreeEncoding;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Problems.Robocode {
  [StorableClass]
  public class TurnLeft : CodeNode {
    public override int MinimumArity { get { return 1; } }
    public override int MaximumArity { get { return 1; } }

    [Storable]
    public override string Prefix { get; set; }

    [Storable]
    public override string Suffix { get; set; }

    [StorableConstructor]
    private TurnLeft(bool deserializing) : base(deserializing) { }
    private TurnLeft(TurnLeft original, Cloner cloner)
      : base(original, cloner) {
    }

    public TurnLeft()
      : base("TurnLeft", "Immediately turns the robot's body to the left by degrees.") {
      this.Prefix = "setTurnLeft(";
      this.Suffix = ");";
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new TurnLeft(this, cloner);
    }

    public override string Interpret(ISymbolicExpressionTreeNode node, System.Collections.Generic.IEnumerable<ISymbolicExpressionTreeNode> children) {
      var enumerator = children.GetEnumerator();

      if (!enumerator.MoveNext()) throw new System.Exception("TurnLeft was not given a child.");
      var symbol = enumerator.Current.Symbol;

      if (!(symbol is Number || symbol is NumericalOperation || symbol is NumericalExpression))
        throw new System.Exception("TurnLeft was given a child of type " + symbol.GetType().ToString() +
            ". The expected child must be of type " + typeof(Number).ToString() + " or " +
            typeof(NumericalOperation).ToString() + " or " +
            typeof(NumericalExpression).ToString() + ".");

      string result = ((CodeNode)symbol).Interpret(enumerator.Current, enumerator.Current.Subtrees);
      if (enumerator.MoveNext()) throw new System.Exception("TurnLeft was given more than one child.");

      return this.Prefix + result + this.Suffix;
    }
  }
}